# Copyright 2015-2016 FUJITSU LIMITED
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from neutron.plugins.ml2 import driver_api

from oslo_log import helpers as log_helpers
from oslo_log import log as logging

from networking_fujitsu._i18n import _LW
from networking_fujitsu.ml2.common import utils as fj_util


LOG = logging.getLogger(__name__)
ISM_DRIVER = 'networking_fujitsu.ml2.ism.ism_base'
_SUPPORTED_NET_TYPES = ['vlan']


class ISMMechanismDriver(driver_api.MechanismDriver):
    """Ml2 Mechanism driver for Fujitsu ISM. """

    def __init__(self):
        self._driver = None
        self.initialize()

    def initialize(self):
        self._driver = ISM_DRIVER

    @classmethod
    def create_ism_base(self, network_type, segmentation_id):

        if (network_type == 'vlan'):
            ism = self._driver.IsmVlanBase(network_type, segmentation_id)
            return ism
        elif (network_type == 'vxlan'):
            ism = self._driver.IsmVxlanBase(network_type, segmentation_id)
            return ism

    @log_helpers.log_method_call
    def create_port_postcommit(self, context):
        """Call ISM API to set VLAN configuration."""

        port = context.current
        if not fj_util.is_baremetal(port):
            LOG.warning("This plugin is under developing and "
                        "not doing at all.")
            return

    @log_helpers.log_method_call
    def update_port_postcommit(self, context):
        """Call ISM API to set VLAN configuration."""

        port = context.current
        if not fj_util.is_baremetal(port):
            LOG.warning("This plugin is under developing and "
                        "not doing at all.")
            return

    @log_helpers.log_method_call
    def delete_port_postcommit(self, context):
        """Call ISM API to reset VLAN configuration."""

        port = context.current
        if not fj_util.is_baremetal(port):
            LOG.warning("This plugin is under developing and "
                        "not doing at all.")
            return

    def validate_network(network):
        """Validate network parameter(network_type and segmentation_id).

        @param a network object
        @return True if network_type is 'VLAN' and segmentation_id is included
                otherwise False
        """

        segment = network.network_segments[0]
        seg_id = segment[driver_api.SEGMENTATION_ID]
        net_type = segment[driver_api.NETWORK_TYPE]
        if (net_type in _SUPPORTED_NET_TYPES and seg_id):
            return True
        LOG.warning(_LW("Only network type vlan is supported."))
        return False



# Copyright 2017 FUJITSU LIMITED
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import mock
import socket

from networking_fujitsu.ml2.common.ovsdb import ovsdb_writer
from networking_fujitsu.ml2.common import tunnel_caller
from networking_fujitsu.ml2.common import type_vxlan
from networking_fujitsu.ml2.fossw import fossw_vxlandriver
from networking_fujitsu.ml2.fossw import mech_fossw
from networking_fujitsu.tests.unit.ml2.common.ovsdb import (
    test_base_connection as base_test)

from neutron.tests import base

from oslo_config import cfg

FOSSW_IPS = ["fake_switch_ip1", "fake_switch_ip2"]
FAKE_SOCKET = base_test.SocketClass(None, None, None, '{"f_key":"f_value"}')


class TestFOSSWVxlanDriver(base.BaseTestCase):
    def setUp(self):
        super(TestFOSSWVxlanDriver, self).setUp()
        cfg.CONF.register_opts(
            mech_fossw.ML2_FUJITSU,
            mech_fossw.ML2_FUJITSU_GROUP
        )
        cfg.CONF.set_override(
            'ovsdb_vlanid_range_min', 2, group='fujitsu_fossw')
        cfg.CONF.set_override('fossw_ips', FOSSW_IPS, group='fujitsu_fossw')
        cfg.CONF.set_override('tenant_network_types', ['vlan', 'vxlan'], 'ml2')

        self.fake_ovsdb_port = 6640
        self.fake_udp_dest_port = 4789
        self.fake_ip_mac_pairs = {'fake_switch_mac1': 'fake_switch_ip1',
                                  'fake_switch_mac2': 'fake_switch_ip2'}
        self.fake_lli = [{'switch_id': 'fake_switch_mac1',
                          'port_id': '0/2',
                          'switch_info': 'fake_switch_name1'}]
        self.fake_llis = [{'switch_id': 'fake_switch_mac1',
                           'port_id': '0/2',
                           'switch_info': 'fake_switch_name1'},
                          {'switch_id': 'fake_switch_mac1',
                           'port_id': '0/3',
                           'switch_info': 'fake_switch_name1'}]
        self.fake_port_context = {'network_id': 'aa-bb-cc',
                                  'mac_address': 'fake_port_mac',
                                  'binding:host_id': 'fake_host_id',
                                  'fixed_ips': [{'ip_address':
                                                 'fake_port_ip1'},
                                                {'ip_address':
                                                 'fake_port_ip2'}]}
        self.fake_switch_ips = ['fake_switch_ip1', 'fake_switch_ip2']
        self.type_vxlan_endpoints = [{'ip_address': 'fake_ip_address1',
                                      'udp_port': 4789,
                                      'host': 'fake_host_name1'},
                                     {'ip_address': 'fake_ip_address2',
                                      'udp_port': 4789,
                                      'host': 'fake_host_name2'}]
        self.fake_context = mock.MagicMock()
        self.fake_request_id = "req-00000000-0000-0000-0000-000000000000"

        with mock.patch.object(socket, 'socket', return_value=FAKE_SOCKET), \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_sw_ep_info',
                              return_value=('fake_endpoint_ip',
                                            'fake_endpoint_hostname')), \
            mock.patch.object(type_vxlan.TypeVxlan, 'get_endpoints',
                              return_value=self.type_vxlan_endpoints), \
            mock.patch.object(fossw_vxlandriver.FOSSWVxlanDriver,
                              '_update_neutron_db',
                              return_value=None):
            self.driver = fossw_vxlandriver.FOSSWVxlanDriver()

    def test_save_all_fossw(self):
        """Test case to test save_all_fossw."""
        self.driver.client = mock.Mock(return_value=None)
        self.assertIsNone(self.driver.save_all_fossw())
        self.assertEqual(len(FOSSW_IPS),
                         self.driver.client.save_running_config.call_count)

    def test_update_neutron_db_insert(self):
        """Test case to test _update_neutron_db.

        In the case that FOS switch endpoint has not been inserted.
        """

        with mock.patch.object(socket, 'socket', return_value=FAKE_SOCKET), \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_sw_ep_info',
                              return_value=('fake_endpoint_ip',
                                            'fake_endpoint_hostname')), \
            mock.patch.object(type_vxlan.TypeVxlan,
                              'get_endpoint_by_ip',
                              return_value={}) as get_epbi, \
            mock.patch.object(type_vxlan.TypeVxlan,
                              'add_endpoint') as add_ep:
            self.driver._update_neutron_db()
            get_epbi.assert_called_with('fake_endpoint_ip')
            add_ep.assert_called_with('fake_endpoint_ip',
                                      'fake_endpoint_hostname', 4789)

    def test_update_neutron_db_update(self):
        """Test case to test _update_neutron_db.

        In the case that FOS switch endpoint hostname has been changed.
        """

        with mock.patch.object(socket, 'socket', return_value=FAKE_SOCKET), \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_sw_ep_info',
                              return_value=('fake_endpoint_ip',
                                            'fake_endpoint_hostname')), \
            mock.patch.object(type_vxlan.TypeVxlan,
                              'get_endpoint_by_ip',
                              return_value={'ip_address': 'fake_endpoint_ip',
                                            'udp_port': 4789,
                                            'host': 'fake_ep_host'}
                              ) as get_epbi, \
            mock.patch.object(type_vxlan.TypeVxlan,
                              'delete_endpoint') as del_ep, \
            mock.patch.object(type_vxlan.TypeVxlan,
                              'add_endpoint') as add_ep:
                self.driver._update_neutron_db()
                get_epbi.assert_called_with('fake_endpoint_ip')
                del_ep.assert_called_with('fake_endpoint_ip')
                add_ep.assert_called_with('fake_endpoint_ip',
                                          'fake_endpoint_hostname', 4789)

    def test_update_neutron_db_none(self):
        """Test case to test _update_neutron_db.

        In the case that FOS switch endpoint has not been inserted.
        """

        with mock.patch.object(socket, 'socket', return_value=FAKE_SOCKET), \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_sw_ep_info',
                              return_value=('fake_endpoint_ip',
                                            'fake_endpoint_hostname')), \
            mock.patch.object(type_vxlan.TypeVxlan,
                              'get_endpoint_by_ip',
                              return_value={'ip_address': 'fake_endpoint_ip',
                                            'udp_port': 4789,
                                            'host': 'fake_endpoint_hostname'}
                              ) as get_epbi:
                self.driver._update_neutron_db()
                get_epbi.assert_called_with('fake_endpoint_ip')

    def test_create_logical_switch(self):
        """Test case to test create_logical_switch."""
        with mock.patch.object(socket, 'socket', return_value=FAKE_SOCKET), \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'insert_logical_switch') as ins_ls, \
            mock.patch.object(fossw_vxlandriver.FOSSWVxlanDriver,
                              'save_all_fossw') as save_all:
            self.driver.create_logical_switch("aa-bb-cc", "fake_vnid")
            ins_ls.assert_called_with("fake_vnid", "aabbcc")
            self.assertEqual(1, save_all.call_count)

    def test_delete_logical_switch(self):
        """Test case to test delete_logical_switch."""
        with mock.patch.object(socket, 'socket', return_value=FAKE_SOCKET), \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_logical_switch_uuid',
                              return_value="fake_uuid") as get_lsuuid, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'delete_logical_switch') as del_ls, \
            mock.patch.object(fossw_vxlandriver.FOSSWVxlanDriver,
                              'save_all_fossw') as save_all:
            self.driver.delete_logical_switch("aa-bb-cc")
            get_lsuuid.assert_called_with("aabbcc")
            del_ls.assert_called_with("fake_uuid")
            self.assertEqual(1, save_all.call_count)

    def test_update_physical_port(self):
        """Test case to test update_physical_port."""
        with mock.patch.object(socket, 'socket', return_value=FAKE_SOCKET), \
            mock.patch.object(type_vxlan.TypeVxlan,
                              'db_get_endpoint_ip_by_host',
                              return_value='fake_target_tunnel_ip'
                              ) as get_epbh, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_logical_switch_uuid',
                              return_value="fake_uuid") as get_ls_uuid, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_binding_vid',
                              return_value=3) as get_bvid, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'update_physical_port') as up_pp, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_ucast_macs_local',
                              return_value=['fake_ucast_macs_local_rows']
                              ) as get_uml, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'delete_ucast_macs_local') as del_uml, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_sw_ep_info',
                              return_value=('fake_locator_ip_local',
                                            'fake_endpoint_hostname')
                              ) as get_sepi, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_physical_locator_uuid',
                              return_value='fake_locator_uuid_local'
                              ) as get_pluuid, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'insert_ucast_macs_local'
                              ) as ins_uml, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'insert_ucast_macs_local_and_locator'
                              ) as ins_umlal, \
            mock.patch.object(fossw_vxlandriver.FOSSWVxlanDriver,
                              '_update_ucast_macs_remote') as _up_umr, \
            mock.patch.object(fossw_vxlandriver.FOSSWVxlanDriver,
                              'save_all_fossw') as _save, \
            mock.patch.object(tunnel_caller.TunnelCaller,
                              'trigger_tunnel_sync') as tun_sync, \
                mock.patch(
                    'networking_fujitsu.ml2.fossw.fossw_vxlandriver.context.'
                    'Context', return_value=self.fake_context):
            self.driver.update_physical_port("fake_vnid", self.fake_lli,
                                             self.fake_port_context,
                                             self.fake_ip_mac_pairs,
                                             req_id=self.fake_request_id)
            get_epbh.assert_called_with('fake_switch_name1')
            get_ls_uuid.assert_called_with("aabbcc")
            get_bvid.assert_called_with("fake_uuid")
            up_pp.assert_called_with("0/2", 3, "fake_uuid")
            get_uml.assert_called_with('fake_port_mac')
            del_uml.assert_called_with('fake_port_mac')
            self.assertEqual(1, get_sepi.call_count)
            get_pluuid.assert_called_with("fake_locator_ip_local")
            ins_uml.assert_called_with("fake_uuid",
                                       "fake_locator_uuid_local",
                                       "fake_port_mac")
            self.assertFalse(ins_umlal.called)
            _up_umr.assert_called_with(
                'fake_switch_ip1', 'aabbcc', 'fake_port_mac',
                'fake_target_tunnel_ip', ['fake_port_ip1', 'fake_port_ip2'])
            self.assertEqual(1, _save.call_count)
            tun_sync.assert_called_with(self.fake_context, 'fake_switch_ip1')

    def test_update_physical_port_without_request_id(self):
        """Test case to test update_physical_port."""
        with mock.patch.object(socket, 'socket', return_value=FAKE_SOCKET), \
            mock.patch.object(type_vxlan.TypeVxlan,
                              'db_get_endpoint_ip_by_host',
                              return_value='fake_target_tunnel_ip'
                              ) as get_epbh, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_logical_switch_uuid',
                              return_value="fake_uuid") as get_ls_uuid, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_binding_vid',
                              return_value=3) as get_bvid, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'update_physical_port') as up_pp, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_ucast_macs_local',
                              return_value=['fake_ucast_macs_local_rows']
                              ) as get_uml, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'delete_ucast_macs_local') as del_uml, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_sw_ep_info',
                              return_value=('fake_locator_ip_local',
                                            'fake_endpoint_hostname')
                              ) as get_sepi, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_physical_locator_uuid',
                              return_value='fake_locator_uuid_local'
                              ) as get_pluuid, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'insert_ucast_macs_local'
                              ) as ins_uml, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'insert_ucast_macs_local_and_locator'
                              ) as ins_umlal, \
            mock.patch.object(fossw_vxlandriver.FOSSWVxlanDriver,
                              '_update_ucast_macs_remote') as _up_umr, \
            mock.patch.object(fossw_vxlandriver.FOSSWVxlanDriver,
                              'save_all_fossw') as _save, \
            mock.patch.object(tunnel_caller.TunnelCaller,
                              'trigger_tunnel_sync') as tun_sync, \
                mock.patch(
                    'networking_fujitsu.ml2.fossw.fossw_vxlandriver.context.'
                    'Context', return_value=self.fake_context):
            self.driver.update_physical_port("fake_vnid", self.fake_lli,
                                             self.fake_port_context,
                                             self.fake_ip_mac_pairs)
            get_epbh.assert_called_with('fake_switch_name1')
            get_ls_uuid.assert_called_with("aabbcc")
            get_bvid.assert_called_with("fake_uuid")
            up_pp.assert_called_with("0/2", 3, "fake_uuid")
            get_uml.assert_called_with('fake_port_mac')
            del_uml.assert_called_with('fake_port_mac')
            self.assertEqual(1, get_sepi.call_count)
            get_pluuid.assert_called_with("fake_locator_ip_local")
            ins_uml.assert_called_with("fake_uuid",
                                       "fake_locator_uuid_local",
                                       "fake_port_mac")
            self.assertFalse(ins_umlal.called)
            _up_umr.assert_called_with(
                'fake_switch_ip1', 'aabbcc', 'fake_port_mac',
                'fake_target_tunnel_ip', ['fake_port_ip1', 'fake_port_ip2'])
            self.assertEqual(1, _save.call_count)
            tun_sync.assert_not_called()

    def test_update_ucast_macs_remote(self):
        """Test case to test _update_ucast_macs_remote."""
        with mock.patch.object(socket, 'socket', return_value=FAKE_SOCKET), \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_logical_switch_uuid',
                              return_value='fake_ls_uuid') as get_lsuuid, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_ucast_macs_remote',
                              return_value=['fake_umr_rows']) as get_umr, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'delete_ucast_macs_remote') as del_umr, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_physical_locator_uuid',
                              return_value='fake_locator_uuid'
                              ) as get_pluuid, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'insert_ucast_macs_remote') as ins_umr, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'insert_ucast_macs_remote_and_locator'
                              ) as ins_umral:

            self.driver._update_ucast_macs_remote(
                'fake_switch_ip1', 'logical_switch_name', 'fake_port_mac',
                'fake_target_tunnel_ip', ['fake_port_ips'])
            get_lsuuid.assert_called_with('logical_switch_name')
            get_umr.assert_called_with('fake_port_mac')
            del_umr.assert_called_with('fake_port_mac')
            get_pluuid.assert_called_with('fake_target_tunnel_ip')
            ins_umr.assert_called_with('fake_ls_uuid', 'fake_port_mac',
                                       ['fake_port_ips'], 'fake_locator_uuid')
            self.assertFalse(ins_umral.called)

    def test_update_ucast_macs_remote_with_locator(self):
        """Test case to test _update_ucast_macs_remote.

        With related locator need to get insert, too.
        """

        with mock.patch.object(socket, 'socket', return_value=FAKE_SOCKET), \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_logical_switch_uuid',
                              return_value='fake_ls_uuid') as get_lsuuid, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_ucast_macs_remote',
                              return_value=['fake_umr_rows']) as get_umr, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'delete_ucast_macs_remote') as del_umr, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'get_physical_locator_uuid',
                              return_value=''
                              ) as get_pluuid, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'insert_ucast_macs_remote') as ins_umr, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'insert_ucast_macs_remote_and_locator'
                              ) as ins_umral:

            self.driver._update_ucast_macs_remote(
                'fake_switch_ip1', 'aabbcc', 'fake_port_mac',
                'fake_target_tunnel_ip', ['fake_port_ips'])
            get_lsuuid.assert_called_with('aabbcc')
            get_umr.assert_called_with('fake_port_mac')
            del_umr.assert_called_with('fake_port_mac')
            get_pluuid.assert_called_with('fake_target_tunnel_ip')
            self.assertFalse(ins_umr.called)
            ins_umral.assert_called_with('fake_ls_uuid', 'fake_port_mac',
                                         ['fake_port_ips'],
                                         'fake_target_tunnel_ip')

    def test_reset_physical_port(self):
        """Test case to test reset_physical_port."""
        with mock.patch.object(socket, 'socket', return_value=FAKE_SOCKET), \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'reset_physical_port') as res_pp, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'delete_ucast_macs_local') as del_uml, \
            mock.patch.object(ovsdb_writer.OVSDBWriter,
                              'delete_ucast_macs_remote') as del_umr, \
            mock.patch.object(fossw_vxlandriver.FOSSWVxlanDriver,
                              'save_all_fossw') as _save:
            self.driver.reset_physical_port(self.fake_lli,
                                            self.fake_port_context,
                                            self.fake_ip_mac_pairs)
            res_pp.assert_called_with("0/2")
            del_uml.assert_called_with("fake_port_mac")
            del_umr.assert_called_with("fake_port_mac")
            self.assertTrue(_save.called)

    def test_update_physical_port_with_lag(self):
        """Test case to test update_physical_port_with_lag."""
        with mock.patch.object(fossw_vxlandriver.FOSSWVxlanDriver,
                               'update_physical_port') as up_pp:
            self.driver.update_physical_port_with_lag('fake_vnid',
                                                      self.fake_llis,
                                                      self.fake_port_context,
                                                      self.fake_ip_mac_pairs,
                                                      self.fake_request_id)
            self.assertEqual(2, up_pp.call_count)
            up_pp.assert_called_with(
                'fake_vnid', mock.ANY, self.fake_port_context,
                self.fake_ip_mac_pairs, self.fake_request_id)

    def test_reset_physical_port_with_lag(self):
        """Test case to test reset_physical_port_with_lag."""
        with mock.patch.object(fossw_vxlandriver.FOSSWVxlanDriver,
                               'reset_physical_port') as res_pp:
            self.driver.reset_physical_port_with_lag(self.fake_llis,
                                                     self.fake_port_context,
                                                     self.fake_ip_mac_pairs)
            self.assertEqual(2, res_pp.call_count)
            res_pp.assert_called_with(
                mock.ANY, self.fake_port_context, self.fake_ip_mac_pairs)



# Copyright 207 FUJITSU LIMITED
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

try:
    from unittest import mock
except ImportError:
    import mock

import copy

from oslo_utils import uuidutils

from neutron.plugins.ml2 import driver_context
from neutron.tests.unit.plugins.ml2 import test_plugin as test_ml2_plugin

LLI = {
    'single': [{
        "switch_id": "00:00:4c:ee:e5:39",
        "port_id": "0/1",
        "switch_info": "dummy_switch1"}],
    'lag': [{
        "switch_id": "00:00:4c:ee:e5:39",
        "port_id": "0/2",
        "switch_info": "dummy_switch1"}, {
        "switch_id": "00:00:4c:ee:e5:39",
        "port_id": "0/3",
        "switch_info": "dummy_switch1"}],
    'mlag': [{
        "switch_id": "00:00:4c:ee:e5:39",
        "port_id": "0/4",
        "switch_info": "dummy_switch1"}, {
        "switch_id": "00:00:4c:ee:e5:40",
        "port_id": "0/4",
        "switch_info": "dummy_switch2"}]
}


class FujitsuMechanismHelper(test_ml2_plugin.Ml2PluginV2TestCase):
    """Fujitsu Mechanism Driver Helper.

    """

    def setUp(self):
        super(FujitsuMechanismHelper, self).setUp()

    def prepare_dummy_context(self, target='port', **kwargs):
        """Prepare dummy PortContext or NetworkContext

        """
        return (self._dummy_port_ctx(**kwargs) if target == 'port' else
                self._dummy_network_ctx(**kwargs))

    def net_seg(self, network_type):
        segment = {'provider:network_type': network_type}
        if network_type == 'flat':
            segment.update({'provider:physical_network': 'physnet1'})
        if network_type == 'vlan':
            segment.update({'provider:segmentation_id': 1111})
            segment.update({'provider:physical_network': 'physnet1'})
        if network_type == 'vxlan':
            segment.update({'provider:segmentation_id': 1111})
        return segment

    def _dummy_port_ctx(self, **kwargs):
        # 'verb', 'port', and 'nic'
        nic = kwargs.get('nic', 'single')
        network_type = kwargs.get('net_type', 'vlan')
        vif_type = kwargs.get('vif_type', None)
        vnic_type = kwargs.get('vnic_type', 'baremetal')
        plugin = mock.Mock()
        plugin_context = mock.Mock()
        binding = mock.Mock()
        project_id = uuidutils.generate_uuid()
        set_original = kwargs.get('set_original', False)

        net_params = {
            'network': {
                'name': 'dummy-net',
                'project_id': project_id,
                'segments': [self.net_seg(network_type)],
            }
        }
        req = self.new_create_request('networks', net_params, self.fmt)
        network = self.deserialize(self.fmt, req.get_response(self.api))
        net = network['network']
        project_id = network['network']['project_id']
        self._create_subnet(self.fmt, net['id'], '172.16.1.0/24')
        baremetal = {
            'binding:vnic_type': vnic_type,
            'binding:host_id': uuidutils.generate_uuid(),
            'binding:profile': {'local_link_information': LLI[nic]}
        }
        data = {
            'port': {
                'network_id': net['id'],
                'project_id': project_id,
                'name': 'dummy-port',
                'admin_state_up': 1,
            }
        }

        req = self.new_create_request('ports', data, self.fmt)
        port = self.deserialize(self.fmt, req.get_response(self.api))
        port_data = port['port']
        port_data.update(baremetal)
        if vif_type:
            port_data.update({'binding:vif_type': vif_type})
        if set_original:
            original_port = copy.deepcopy(port_data)
            original_port.update({'binding:vif_type': 'other'})
        else:
            original_port = None
        with mock.patch.object(driver_context.segments_db,
                               'get_network_segments') as segments:
            segments.return_value = [self.net_seg(network_type)]
            mock_ctx = driver_context.PortContext(
                plugin, plugin_context, port_data, net, binding, None,
                original_port=original_port)
            mock_ctx._segments_to_bind = [mock_ctx.network.current]
            return mock_ctx

    def _dummy_network_ctx(self, **kwargs):
        """Prepare Dummy NetworkContext

        """
        network_type = kwargs.get('net_type', 'vlan')
        project_id = uuidutils.generate_uuid()
        net_params = {
            'network': {
                'tenant_id': project_id,
                'ipv4_address_scope': None,
                'admin_state_up': True,
                'name': 'dummy-net',
                'project_id': project_id,
                'status': 'ACTIVE',
                'description': '',
                'router:external': False,
                'subnets': [],
                'shared': False,
                'availability_zone_hints': [],
                'ipv6_address_scope': None,
                'availability_zones': [],
                'mtu': 1500,
                'id': '75959a8b-8321-41f9-b11f-f15fcb47b8e3'
            }
        }
        net_params['network'].update(self.net_seg(network_type))
        net = net_params['network']
        plugin = mock.Mock()
        plugin_context = mock.Mock()
        with mock.patch.object(driver_context.segments_db,
                               'get_network_segments'):
            return driver_context.NetworkContext(plugin, plugin_context, net)
